/** Rill(parse-url) v0.5.2 */
"use strict";
var URL, dateReg, httpReg, parseDates, qSet, qs, unflattenQuery;

URL = require("url");

qs = require("querystring");

qSet = require("q-set");

httpReg = /^https?:\/\//i;

dateReg = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*))(?:Z|(\+|-)([\d|:]*))?$/;


/*
 * When used as a transform for JSON.parse date strings will be turned into date objects.
#
 * @param {String} key
 * @param {String} val
 */

parseDates = function(key, val) {
  if ("string" === typeof val && dateReg.test(val)) {
    return new Date(val);
  } else {
    return val;
  }
};


/*
 * Utility to convert nested querystring fields to a nested object.
#
 * @param {Object} obj
 * @return {Object}
 */

unflattenQuery = function(obj) {
  var key, result, val;
  result = {};
  for (key in obj) {
    val = obj[key];
    qSet(result, key, parseDates(null, val));
  }
  return result;
};


/*
 * Extract a parsed url from a request object.
#
 * @param {Request} req
 */

module.exports = function(url, req) {
  var defaults, hostname, origin, parsed, port, protocol, ref, ref1;
  if (url == null) {
    url = "/";
  }
  defaults = (req ? ((ref = (req.headers["x-forwarded-host"] || req.headers["host"] || "").split(":"), hostname = ref[0], port = ref[1], ref), protocol = ((ref1 = req.socket) != null ? ref1.encrypted : void 0) ? "https:" : "http:", {
    hostname: hostname,
    port: port,
    protocol: protocol
  }) : {});
  protocol = (defaults.protocol ? (":" === defaults.protocol.slice(-1) ? defaults.protocol.slice(0, -1) : defaults.protocol) : "http");
  port = (defaults.port ? ":" + defaults.port : "");
  url = (httpReg.test(url) ? url : protocol + "://" + defaults.hostname + port + url);
  parsed = URL.parse(url);
  parsed.search = decodeURIComponent(parsed.query || "");
  if (parsed.search) {
    try {
      parsed.query = (parsed.search[0] === "{" && parsed.search[parsed.search.length - 1] === "}" ? JSON.parse(parsed.search, parseDates) : unflattenQuery(qs.parse(parsed.search)));
    } catch (_error) {}
  }
  if (typeof parsed.query !== "object") {
    parsed.query = {};
  }
  origin = parsed.protocol + "//" + parsed.hostname;
  if (parsed.port) {
    origin += ":" + parsed.port;
  }
  return {
    url: url,
    origin: origin,
    protocol: parsed.protocol.slice(0, -1),
    host: parsed.host || "",
    hostname: parsed.hostname || "",
    port: parsed.port || "",
    path: parsed.pathname || "",
    search: parsed.search || "",
    query: parsed.query || {},
    hash: (parsed.hash || "").slice(1)
  };
};
