URL     = require("url")
qs      = require("querystring")
qSet    = require("q-set")
httpReg = /^https?:\/\//i
dateReg = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*))(?:Z|(\+|-)([\d|:]*))?$/

###
# When used as a transform for JSON.parse date strings will be turned into date objects.
#
# @param {String} key
# @param {String} val
###
parseDates = (key, val)->
	if "string" is typeof val and dateReg.test(val) then new Date(val)
	else val

###
# Utility to convert nested querystring fields to a nested object.
#
# @param {Object} obj
# @return {Object}
###
unflattenQuery = (obj)->
	result = {}
	qSet(result, key, parseDates(null, val)) for key, val of obj
	result

###
# Extract a parsed url from a request object.
#
# @param {Request} req
###
module.exports = (url = "/", req)->
	defaults = (
		if req
			[hostname, port] = (req.headers["x-forwarded-host"] or req.headers["host"] or "").split(":")
			protocol         = if req.socket?.encrypted then "https:" else "http:"
			{ hostname, port, protocol }
		else {}
	)

	protocol  = (
		if defaults.protocol then (
			if ":" is defaults.protocol[-1...] then defaults.protocol[0...-1]
			else defaults.protocol
		) else "http"
	)
	port = (
		if defaults.port then ":" + defaults.port
		else ""
	)
	url = (
		if httpReg.test(url) then url
		else "#{protocol}://#{defaults.hostname}#{port}#{url}"
	)
	parsed        = URL.parse(url)
	parsed.search = decodeURIComponent(parsed.query or "")

	# Check if it is a json query.
	if parsed.search then try parsed.query = (
		if parsed.search[0] is "{" and parsed.search[parsed.search.length - 1] is "}"
			JSON.parse(parsed.search, parseDates)
		else
			unflattenQuery(qs.parse(parsed.search))
	)

	parsed.query = {} unless typeof parsed.query is "object"

	origin = "#{parsed.protocol}//#{parsed.hostname}"
	origin += ":#{parsed.port}" if parsed.port

	url:      url
	origin:   origin
	protocol: parsed.protocol[0...-1]
	host:     (parsed.host or "")
	hostname: (parsed.hostname or "")
	port:     (parsed.port or "")
	path:     (parsed.pathname or "")
	search:   (parsed.search or "")
	query:    (parsed.query or {})
	hash:     (parsed.hash or "")[1...]