should   = require("should")
parseURL = require("../src/index.coffee")
details  = require("../package.json")

describe details.name, ->
	it "should parse a valid url", ->
		protocol    = "https"
		host        = "test.com"
		subdomains  = "a.b"
		port        = 3000
		path        = "path"
		hash        = "test"
		query       = JSON.stringify({ test: yes })

		req = parseURL("#{protocol}://#{subdomains}.#{host}:#{port}/#{path}?#{query}##{hash}")
		req.should
			.eql(
				url:      "https://a.b.test.com:3000/path?{\"test\":true}#test"
				protocol: "https"
				host:     "a.b.test.com:3000"
				hostname: "a.b.test.com"
				port:     "3000"
				path:     "/path"
				search:   "{\"test\":true}",
				query:    { test: true }
				hash:     "test"
				origin:   "https://a.b.test.com:3000"
			)

	it "should use default protocol, host and port", ->
		path    = "path"
		hash    = "test"
		query   = JSON.stringify({ test: yes })

		req = parseURL("/#{path}?#{query}##{hash}",
			headers: host: "test.com:3002"
			socket: encrypted: true
		)

		req.should
			.have.properties(
				protocol: "https"
				hostname: "test.com"
				port:     "3002"
			)

	it "should default to http and no port without defaults", ->
		path    = "path"
		hash    = "test"
		query   = JSON.stringify({ test: yes })

		req = parseURL("/#{path}?#{query}##{hash}")
		req.should
			.have.properties(
				protocol: "http"
				hostname: "undefined"
				hash:     hash
				port:     ""
			)